# pipelines/downsampling.py
from ..preprocessing import split_holdout, scale, cluster_downsample
import pandas as pd
from pathlib import Path

ROOT = Path(__file__).resolve().parents[2]
CACHED_DIR = ROOT / "data" / "downsampling"
CACHED_DIR.mkdir(parents=True, exist_ok=True)

def prepare(df, model_name):
    (X_train_raw, y_train_raw,
     X_test_alz, y_test_alz, df_test_alz,
     X_test_esq, y_test_esq, df_test_esq) = split_holdout(df)

    # Escalado
    if model_name in {"knn", "logistic_regression"}:
        X_train_scaled, X_test_alz = scale(X_train_raw, X_test_alz)
        _,              X_test_esq = scale(X_train_raw, X_test_esq)
    else:
        X_train_scaled = X_train_raw.copy()

    x_path = CACHED_DIR / "X_train_downsampled.parquet"
    y_path = CACHED_DIR / "y_train_downsampled.parquet"

    if x_path.exists() and y_path.exists():
        X_train = pd.read_parquet(x_path)
        y_train = pd.read_parquet(y_path).squeeze()
    else:
        # SMOTE sobre datos ya escalados
        X_train, y_train = cluster_downsample(X_train_scaled, y_train_raw)
        X_train.to_parquet(x_path, index=False)
        pd.DataFrame(y_train, columns=["class"]).to_parquet(y_path, index=False)

    return X_train, y_train, X_test_alz, y_test_alz, df_test_alz, X_test_esq, y_test_esq, df_test_esq
